@@ -10,8 +10,9 @@ import logging
 import random
 import re
 import sys
+from collections.abc import Callable
 from dataclasses import dataclass, field
-from typing import Any, Callable, Literal, Optional, Union
+from typing import Any, Literal
 
 from ..code_utils import content_str
 from ..doc_utils import export_module
@@ -30,8 +31,7 @@ from ..events.agent_events import (
 from ..exception_utils import AgentNameConflictError, NoEligibleSpeakerError, UndefinedNextAgentError
 from ..graph_utils import check_graph_validity, invert_disallowed_to_allowed
 from ..io.base import IOStream
-from ..llm_config import LLMConfig
-from ..oai.client import ModelClient
+from ..llm_config import LLMConfig, ModelClient
 from ..runtime_logging import log_new_agent, logging_enabled
 from .agent import Agent
 from .contrib.capabilities import transform_messages
@@ -134,10 +134,10 @@ class GroupChat:
     max_round: int = 10
     admin_name: str = "Admin"
     func_call_filter: bool = True
-    speaker_selection_method: Union[Literal["auto", "manual", "random", "round_robin"], Callable[..., Any]] = "auto"
+    speaker_selection_method: Literal["auto", "manual", "random", "round_robin"] | Callable[..., Any] = "auto"
     max_retries_for_selecting_speaker: int = 2
-    allow_repeat_speaker: Optional[Union[bool, list[Agent]]] = None
-    allowed_or_disallowed_speaker_transitions: Optional[dict[str, Any]] = None
+    allow_repeat_speaker: bool | list[Agent] | None = None
+    allowed_or_disallowed_speaker_transitions: dict[str, Any] | None = None
     speaker_transitions_type: Literal["allowed", "disallowed", None] = None
     enable_clear_history: bool = False
     send_introductions: bool = False
@@ -159,11 +159,11 @@ class GroupChat:
     The names are case-sensitive and should not be abbreviated or changed.
     The only names that are accepted are {agentlist}.
     Respond with ONLY the name of the speaker and DO NOT provide a reason."""
-    select_speaker_transform_messages: Optional[transform_messages.TransformMessages] = None
-    select_speaker_auto_verbose: Optional[bool] = False
-    select_speaker_auto_model_client_cls: Optional[Union[ModelClient, list[ModelClient]]] = None
-    select_speaker_auto_llm_config: Optional[Union[LLMConfig, dict[str, Any], Literal[False]]] = None
-    role_for_select_speaker_messages: Optional[str] = "system"
+    select_speaker_transform_messages: transform_messages.TransformMessages | None = None
+    select_speaker_auto_verbose: bool | None = False
+    select_speaker_auto_model_client_cls: ModelClient | list[ModelClient] | None = None
+    select_speaker_auto_llm_config: LLMConfig | dict[str, Any] | Literal[False] | None = None
+    role_for_select_speaker_messages: str | None = "system"
 
     _VALID_SPEAKER_SELECTION_METHODS = ["auto", "manual", "random", "round_robin"]
     _VALID_SPEAKER_TRANSITIONS_TYPE = ["allowed", "disallowed", None]
@@ -315,9 +315,7 @@ class GroupChat:
         message["content"] = content_str(message["content"])
         self.messages.append(message)
 
-    def agent_by_name(
-        self, name: str, recursive: bool = False, raise_on_name_conflict: bool = False
-    ) -> Optional[Agent]:
+    def agent_by_name(self, name: str, recursive: bool = False, raise_on_name_conflict: bool = False) -> Agent | None:
         """Returns the agent with a given name. If recursive is True, it will search in nested teams."""
         agents = self.nested_agents() if recursive else self.agents
         filtered_agents = [agent for agent in agents if agent.name == name]
@@ -336,7 +334,7 @@ class GroupChat:
                 agents.extend(agent.groupchat.nested_agents())
         return agents
 
-    def next_agent(self, agent: Agent, agents: Optional[list[Agent]] = None) -> Agent:
+    def next_agent(self, agent: Agent, agents: list[Agent] | None = None) -> Agent:
         """Return the next agent in the list."""
         if agents is None:
             agents = self.agents
@@ -360,7 +358,7 @@ class GroupChat:
         # Explicitly handle cases where no valid next agent exists in the provided subset.
         raise UndefinedNextAgentError()
 
-    def select_speaker_msg(self, agents: Optional[list[Agent]] = None) -> str:
+    def select_speaker_msg(self, agents: list[Agent] | None = None) -> str:
         """Return the system message for selecting the next speaker. This is always the *first* message in the context."""
         if agents is None:
             agents = self.agents
@@ -371,7 +369,7 @@ class GroupChat:
         return_msg = self.select_speaker_message_template.format(roles=roles, agentlist=agentlist)
         return return_msg
 
-    def select_speaker_prompt(self, agents: Optional[list[Agent]] = None) -> str:
+    def select_speaker_prompt(self, agents: list[Agent] | None = None) -> str:
         """Return the floating system prompt selecting the next speaker.
         This is always the *last* message in the context.
         Will return None if the select_speaker_prompt_template is None.
@@ -387,7 +385,7 @@ class GroupChat:
         return_prompt = f"{self.select_speaker_prompt_template}".replace("{agentlist}", agentlist)
         return return_prompt
 
-    def introductions_msg(self, agents: Optional[list[Agent]] = None) -> str:
+    def introductions_msg(self, agents: list[Agent] | None = None) -> str:
         """Return the system message for selecting the next speaker. This is always the *first* message in the context."""
         if agents is None:
             agents = self.agents
@@ -398,7 +396,7 @@ class GroupChat:
 
         return f"{intro_msg}\n\n{participant_roles}"
 
-    def manual_select_speaker(self, agents: Optional[list[Agent]] = None) -> Union[Agent, None]:
+    def manual_select_speaker(self, agents: list[Agent] | None = None) -> Agent | None:
         """Manually select the next speaker."""
         iostream = IOStream.get_default()
 
@@ -429,7 +427,7 @@ class GroupChat:
                 iostream.send(SelectSpeakerInvalidInputEvent(agents=agents))
         return None
 
-    def random_select_speaker(self, agents: Optional[list[Agent]] = None) -> Union[Agent, None]:
+    def random_select_speaker(self, agents: list[Agent] | None = None) -> Agent | None:
         """Randomly select the next speaker."""
         if agents is None:
             agents = self.agents
@@ -438,7 +436,7 @@ class GroupChat:
     def _prepare_and_select_agents(
         self,
         last_speaker: Agent,
-    ) -> tuple[Optional[Agent], list[Agent], Optional[list[dict[str, Any]]]]:
+    ) -> tuple[Agent | None, list[Agent], list[dict[str, Any]] | None]:
         # If self.speaker_selection_method is a callable, call it to get the next speaker.
         # If self.speaker_selection_method is a string, return it.
         speaker_selection_method = self.speaker_selection_method
@@ -577,7 +575,7 @@ class GroupChat:
             return self.next_agent(last_speaker)
 
         # auto speaker selection with 2-agent chat
-        return self._auto_select_speaker(last_speaker, selector, messages, agents)
+        return self._auto_select_speaker(last_speaker, selector, messages if messages else self.messages, agents)
 
     async def a_select_speaker(self, last_speaker: Agent, selector: ConversableAgent) -> Agent:
         """Select the next speaker (with requery), asynchronously."""
@@ -589,9 +587,9 @@ class GroupChat:
             return self.next_agent(last_speaker)
 
         # auto speaker selection with 2-agent chat
-        return await self.a_auto_select_speaker(last_speaker, selector, messages, agents)
+        return await self.a_auto_select_speaker(last_speaker, selector, messages if messages else self.messages, agents)
 
-    def _finalize_speaker(self, last_speaker: Agent, final: bool, name: str, agents: Optional[list[Agent]]) -> Agent:
+    def _finalize_speaker(self, last_speaker: Agent, final: bool, name: str, agents: list[Agent] | None) -> Agent:
         if not final:
             # the LLM client is None, thus no reply is generated. Use round robin instead.
             return self.next_agent(last_speaker, agents)
@@ -653,7 +651,7 @@ class GroupChat:
             self._register_client_from_config(agent, self.select_speaker_auto_llm_config)
 
     def _create_internal_agents(
-        self, agents, max_attempts, messages, validate_speaker_name, selector: Optional[ConversableAgent] = None
+        self, agents, max_attempts, messages, validate_speaker_name, selector: ConversableAgent | None = None
     ):
         checking_agent = ConversableAgent("checking_agent", default_auto_reply=max_attempts)
 
@@ -691,8 +689,8 @@ class GroupChat:
         self,
         last_speaker: Agent,
         selector: ConversableAgent,
-        messages: Optional[list[dict[str, Any]]],
-        agents: Optional[list[Agent]],
+        messages: list[dict[str, Any]] | None,
+        agents: list[Agent] | None,
     ) -> Agent:
         """Selects next speaker for the "auto" speaker selection method. Utilises its own two-agent chat to determine the next speaker and supports requerying.
 
@@ -726,9 +724,7 @@ class GroupChat:
         attempt = 0
 
         # Registered reply function for checking_agent, checks the result of the response for agent names
-        def validate_speaker_name(
-            recipient, messages, sender, config
-        ) -> tuple[bool, Optional[Union[str, dict[str, Any]]]]:
+        def validate_speaker_name(recipient, messages, sender, config) -> tuple[bool, str | dict[str, Any] | None]:
             # The number of retries left, starting at max_retries_for_selecting_speaker
             nonlocal attempts_left
             nonlocal attempt
@@ -776,8 +772,8 @@ class GroupChat:
         self,
         last_speaker: Agent,
         selector: ConversableAgent,
-        messages: Optional[list[dict[str, Any]]],
-        agents: Optional[list[Agent]],
+        messages: list[dict[str, Any]] | None,
+        agents: list[Agent] | None,
     ) -> Agent:
         """(Asynchronous) Selects next speaker for the "auto" speaker selection method. Utilises its own two-agent chat to determine the next speaker and supports requerying.
 
@@ -810,9 +806,7 @@ class GroupChat:
         attempt = 0
 
         # Registered reply function for checking_agent, checks the result of the response for agent names
-        def validate_speaker_name(
-            recipient, messages, sender, config
-        ) -> tuple[bool, Optional[Union[str, dict[str, Any]]]]:
+        def validate_speaker_name(recipient, messages, sender, config) -> tuple[bool, str | dict[str, Any] | None]:
             # The number of retries left, starting at max_retries_for_selecting_speaker
             nonlocal attempts_left
             nonlocal attempt
@@ -857,21 +851,22 @@ class GroupChat:
 
     def _validate_speaker_name(
         self, recipient, messages, sender, config, attempts_left, attempt, agents
-    ) -> tuple[bool, Optional[Union[str, dict[str, Any]]]]:
+    ) -> tuple[bool, str | dict[str, Any] | None]:
         """Validates the speaker response for each round in the internal 2-agent
         chat within the  auto select speaker method.
 
         Used by auto_select_speaker and a_auto_select_speaker.
         """
         # Validate the speaker name selected
-        select_name = messages[-1]["content"].strip()
-
-        mentions = self._mentioned_agents(select_name, agents)
+        if messages and (name := messages[-1].get("content")):
+            mentions = self._mentioned_agents(name.strip(), agents)
+        else:
+            mentions = []
+        no_of_mentions = len(mentions)
 
         # Output the query and requery results
         if self.select_speaker_auto_verbose:
             iostream = IOStream.get_default()
-            no_of_mentions = len(mentions)
             if no_of_mentions == 1:
                 # Success on retry, we have just one name mentioned
                 iostream.send(
@@ -901,14 +896,14 @@ class GroupChat:
                     )
                 )
 
-        if len(mentions) == 1:
+        if no_of_mentions == 1:
             # Success on retry, we have just one name mentioned
             selected_agent_name = next(iter(mentions))
 
             # Add the selected agent to the response so we can return it
             messages.append({"role": "user", "content": f"[AGENT SELECTED]{selected_agent_name}"})
 
-        elif len(mentions) > 1:
+        elif no_of_mentions > 1:
             # More than one name on requery so add additional reminder prompt for next retry
 
             if attempts_left:
@@ -948,7 +943,7 @@ class GroupChat:
 
         return True, None
 
-    def _process_speaker_selection_result(self, result, last_speaker: ConversableAgent, agents: Optional[list[Agent]]):
+    def _process_speaker_selection_result(self, result, last_speaker: ConversableAgent, agents: list[Agent] | None):
         """Checks the result of the auto_select_speaker function, returning the
         agent to speak.
 
@@ -983,7 +978,7 @@ class GroupChat:
             roles.append(f"{agent.name}: {agent.description}".strip())
         return "\n".join(roles)
 
-    def _mentioned_agents(self, message_content: Union[str, list], agents: Optional[list[Agent]]) -> dict:
+    def _mentioned_agents(self, message_content: str | list, agents: list[Agent] | None) -> dict:
         """Counts the number of times each agent is mentioned in the provided message content.
         Agent names will match under any of the following conditions (all case-sensitive):
         - Exact name match
@@ -1023,6 +1018,40 @@ class GroupChat:
                 mentions[agent.name] = count
         return mentions
 
+    def _run_input_guardrails(
+        self,
+        agent: "ConversableAgent",
+        messages: list[dict[str, Any]] | None = None,
+    ) -> str | None:
+        """Run input guardrails for an agent before the reply is generated.
+
+        Args:
+            agent (ConversableAgent): The agent whose input guardrails to run.
+            messages (Optional[list[dict[str, Any]]]): The messages to check against the guardrails.
+        """
+        for guardrail in agent.input_guardrails:
+            guardrail_result = guardrail.check(context=messages)
+
+            if guardrail_result.activated:
+                guardrail.target.activate_target(self)
+                return f"{guardrail.activation_message}\nJustification: {guardrail_result.justification}"
+        return None
+
+    def _run_output_guardrails(self, agent: "ConversableAgent", reply: str) -> None:
+        """Run output guardrails for an agent after the reply is generated.
+
+        Args:
+            agent (ConversableAgent): The agent whose output guardrails to run.
+            reply (str): The reply generated by the agent.
+        """
+        for guardrail in agent.output_guardrails:
+            guardrail_result = guardrail.check(context=reply)
+
+            if guardrail_result.activated:
+                guardrail.target.activate_target(self)
+                return f"{guardrail.activation_message}\nJustification: {guardrail_result.justification}"
+        return None
+
 
 @export_module("autogen")
 class GroupChatManager(ConversableAgent):
@@ -1031,11 +1060,11 @@ class GroupChatManager(ConversableAgent):
     def __init__(
         self,
         groupchat: GroupChat,
-        name: Optional[str] = "chat_manager",
+        name: str | None = "chat_manager",
         # unlimited consecutive auto reply by default
-        max_consecutive_auto_reply: Optional[int] = sys.maxsize,
+        max_consecutive_auto_reply: int | None = sys.maxsize,
         human_input_mode: Literal["ALWAYS", "NEVER", "TERMINATE"] = "NEVER",
-        system_message: Optional[Union[str, list]] = "Group chat manager.",
+        system_message: str | list | None = "Group chat manager.",
         silent: bool = False,
         **kwargs: Any,
     ):
@@ -1145,10 +1174,10 @@ class GroupChatManager(ConversableAgent):
 
     def run_chat(
         self,
-        messages: Optional[list[dict[str, Any]]] = None,
-        sender: Optional[Agent] = None,
-        config: Optional[GroupChat] = None,
-    ) -> tuple[bool, Optional[str]]:
+        messages: list[dict[str, Any]] | None = None,
+        sender: Agent | None = None,
+        config: GroupChat | None = None,
+    ) -> tuple[bool, str | None]:
         """Run a group chat."""
         iostream = IOStream.get_default()
 
@@ -1194,8 +1223,17 @@ class GroupChatManager(ConversableAgent):
                 if not silent:
                     iostream = IOStream.get_default()
                     iostream.send(GroupChatRunChatEvent(speaker=speaker, silent=silent))
-                # let the speaker speak
-                reply = speaker.generate_reply(sender=self)
+
+                guardrails_activated = False
+                guardrails_reply = groupchat._run_input_guardrails(speaker, speaker._oai_messages[self])
+
+                if guardrails_reply is not None:
+                    # if a guardrail has been activated, then the next target has been set and the guardrail reply will be sent
+                    guardrails_activated = True
+                    reply = guardrails_reply
+                else:
+                    # let the speaker speak
+                    reply = speaker.generate_reply(sender=self)
             except KeyboardInterrupt:
                 # let the admin agent speak if interrupted
                 if groupchat.admin_name in groupchat.agent_names:
@@ -1215,6 +1253,15 @@ class GroupChatManager(ConversableAgent):
                 termination_reason = "No reply generated"
                 break
 
+            if not guardrails_activated:
+                # if the input guardrails were not activated, and the agent returned a reply
+                guardrails_reply = groupchat._run_output_guardrails(speaker, reply)
+
+                if guardrails_reply is not None:
+                    # if a guardrail has been activated, then the next target has been set and the guardrail reply will be sent
+                    guardrails_activated = True
+                    reply = guardrails_reply
+
             # check for "clear history" phrase in reply and activate clear history function if found
             if (
                 groupchat.enable_clear_history
@@ -1233,15 +1280,19 @@ class GroupChatManager(ConversableAgent):
                 a.previous_cache = None
 
         if termination_reason:
-            iostream.send(TerminationEvent(termination_reason=termination_reason))
+            iostream.send(
+                TerminationEvent(
+                    termination_reason=termination_reason, sender=self, recipient=speaker if speaker else None
+                )
+            )
 
         return True, None
 
     async def a_run_chat(
         self,
-        messages: Optional[list[dict[str, Any]]] = None,
-        sender: Optional[Agent] = None,
-        config: Optional[GroupChat] = None,
+        messages: list[dict[str, Any]] | None = None,
+        sender: Agent | None = None,
+        config: GroupChat | None = None,
     ):
         """Run a group chat asynchronously."""
         iostream = IOStream.get_default()
@@ -1287,8 +1338,19 @@ class GroupChatManager(ConversableAgent):
             try:
                 # select the next speaker
                 speaker = await groupchat.a_select_speaker(speaker, self)
-                # let the speaker speak
-                reply = await speaker.a_generate_reply(sender=self)
+                if not silent:
+                    iostream.send(GroupChatRunChatEvent(speaker=speaker, silent=silent))
+
+                guardrails_activated = False
+                guardrails_reply = groupchat._run_input_guardrails(speaker, speaker._oai_messages[self])
+
+                if guardrails_reply is not None:
+                    # if a guardrail has been activated, then the next target has been set and the guardrail reply will be sent
+                    guardrails_activated = True
+                    reply = guardrails_reply
+                else:
+                    # let the speaker speak
+                    reply = await speaker.a_generate_reply(sender=self)
             except KeyboardInterrupt:
                 # let the admin agent speak if interrupted
                 if groupchat.admin_name in groupchat.agent_names:
@@ -1308,6 +1370,24 @@ class GroupChatManager(ConversableAgent):
                 termination_reason = "No reply generated"
                 break
 
+            if not guardrails_activated:
+                # if the input guardrails were not activated, and the agent returned a reply
+                guardrails_reply = groupchat._run_output_guardrails(speaker, reply)
+
+                if guardrails_reply is not None:
+                    # if a guardrail has been activated, then the next target has been set and the guardrail reply will be sent
+                    guardrails_activated = True
+                    reply = guardrails_reply
+
+            # check for "clear history" phrase in reply and activate clear history function if found
+            if (
+                groupchat.enable_clear_history
+                and isinstance(reply, dict)
+                and reply["content"]
+                and "CLEAR HISTORY" in reply["content"].upper()
+            ):
+                reply["content"] = self.clear_agents_history(reply, groupchat)
+
             # The speaker sends the message without requesting a reply
             await speaker.a_send(reply, self, request_reply=False, silent=silent)
             message = self.last_message(speaker)
@@ -1317,15 +1397,19 @@ class GroupChatManager(ConversableAgent):
                 a.previous_cache = None
 
         if termination_reason:
-            iostream.send(TerminationEvent(termination_reason=termination_reason))
+            iostream.send(
+                TerminationEvent(
+                    termination_reason=termination_reason, sender=self, recipient=speaker if speaker else None
+                )
+            )
 
         return True, None
 
     def resume(
         self,
-        messages: Union[list[dict[str, Any]], str],
-        remove_termination_string: Optional[Union[str, Callable[[str], str]]] = None,
-        silent: Optional[bool] = False,
+        messages: list[dict[str, Any]] | str,
+        remove_termination_string: str | Callable[[str], str] | None = None,
+        silent: bool | None = False,
     ) -> tuple[ConversableAgent, dict[str, Any]]:
         """Resumes a group chat using the previous messages as a starting point. Requires the agents, group chat, and group chat manager to be established
         as per the original group chat.
@@ -1426,9 +1510,9 @@ class GroupChatManager(ConversableAgent):
 
     async def a_resume(
         self,
-        messages: Union[list[dict[str, Any]], str],
-        remove_termination_string: Optional[Union[str, Callable[[str], str]]] = None,
-        silent: Optional[bool] = False,
+        messages: list[dict[str, Any]] | str,
+        remove_termination_string: str | Callable[[str], str] | None = None,
+        silent: bool | None = False,
     ) -> tuple[ConversableAgent, dict[str, Any]]:
         """Resumes a group chat using the previous messages as a starting point, asynchronously. Requires the agents, group chat, and group chat manager to be established
         as per the original group chat.
@@ -1489,10 +1573,10 @@ class GroupChatManager(ConversableAgent):
                 for agent in self._groupchat.agents:
                     if agent.name == message["name"]:
                         # An agent`s message is sent to the Group Chat Manager
-                        agent.a_send(message, self, request_reply=False, silent=True)
+                        await agent.a_send(message, self, request_reply=False, silent=True)
                     else:
                         # Otherwise, messages are sent from the Group Chat Manager to the agent
-                        self.a_send(message, agent, request_reply=False, silent=True)
+                        await self.a_send(message, agent, request_reply=False, silent=True)
 
                 # Add previous message to the new groupchat, if it's an admin message the name may not match so add the message directly
                 if message_speaker_agent:
@@ -1552,7 +1636,7 @@ class GroupChatManager(ConversableAgent):
                 raise Exception(f"Agent name in message doesn't exist as agent in group chat: {message['name']}")
 
     def _process_resume_termination(
-        self, remove_termination_string: Union[str, Callable[[str], str]], messages: list[dict[str, Any]]
+        self, remove_termination_string: str | Callable[[str], str], messages: list[dict[str, Any]]
     ):
         """Removes termination string, if required, and checks if termination may occur.
 
