@@ -3,8 +3,9 @@
 # SPDX-License-Identifier: Apache-2.0
 
 from abc import ABC
+from collections.abc import Callable
 from copy import deepcopy
-from typing import TYPE_CHECKING, Any, Callable, Literal, Optional, Union
+from typing import TYPE_CHECKING, Any, Literal, Union
 from uuid import UUID
 
 from pydantic import BaseModel, field_validator, model_serializer
@@ -56,22 +57,22 @@ EventRole = Literal["assistant", "function", "tool"]
 
 
 class BasePrintReceivedEvent(BaseEvent, ABC):
-    content: Union[str, int, float, bool]
+    content: str | int | float | bool
     sender: str
     recipient: str
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
         f(f"{colored(self.sender, 'yellow')} (to {self.recipient}):\n", flush=True)
 
 
 @wrap_event
 class FunctionResponseEvent(BasePrintReceivedEvent):
-    name: Optional[str] = None
+    name: str | None = None
     role: EventRole = "function"
-    content: Union[str, int, float, bool]
+    content: str | int | float | bool
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
         super().print(f)
 
@@ -85,11 +86,11 @@ class FunctionResponseEvent(BasePrintReceivedEvent):
 
 
 class ToolResponse(BaseModel):
-    tool_call_id: Optional[str] = None
+    tool_call_id: str | None = None
     role: EventRole = "tool"
-    content: Union[str, int, float, bool]
+    content: str | int | float | bool
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
         id = self.tool_call_id or "No id found"
         tool_print = f"***** Response from calling {self.role} ({id}) *****"
@@ -102,9 +103,9 @@ class ToolResponse(BaseModel):
 class ToolResponseEvent(BasePrintReceivedEvent):
     role: EventRole = "tool"
     tool_responses: list[ToolResponse]
-    content: Union[str, int, float, bool]
+    content: str | int | float | bool
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
         super().print(f)
 
@@ -114,10 +115,10 @@ class ToolResponseEvent(BasePrintReceivedEvent):
 
 
 class FunctionCall(BaseModel):
-    name: Optional[str] = None
-    arguments: Optional[str] = None
+    name: str | None = None
+    arguments: str | None = None
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
 
         name = self.name or "(No function name found)"
@@ -136,10 +137,10 @@ class FunctionCall(BaseModel):
 
 @wrap_event
 class FunctionCallEvent(BasePrintReceivedEvent):
-    content: Optional[Union[str, int, float, bool]] = None  # type: ignore [assignment]
+    content: str | int | float | bool | None = None  # type: ignore [assignment]
     function_call: FunctionCall
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
         super().print(f)
 
@@ -152,11 +153,11 @@ class FunctionCallEvent(BasePrintReceivedEvent):
 
 
 class ToolCall(BaseModel):
-    id: Optional[str] = None
+    id: str | None = None
     function: FunctionCall
     type: str
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
 
         id = self.id or "No tool call id found"
@@ -177,14 +178,14 @@ class ToolCall(BaseModel):
 
 @wrap_event
 class ToolCallEvent(BasePrintReceivedEvent):
-    content: Optional[Union[str, int, float, bool]] = None  # type: ignore [assignment]
-    refusal: Optional[str] = None
-    role: Optional[EventRole] = None
-    audio: Optional[str] = None
-    function_call: Optional[FunctionCall] = None
+    content: str | int | float | bool | None = None  # type: ignore [assignment]
+    refusal: str | None = None
+    role: EventRole | None = None
+    audio: str | None = None
+    function_call: FunctionCall | None = None
     tool_calls: list[ToolCall]
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
         super().print(f)
 
@@ -199,7 +200,7 @@ class ToolCallEvent(BasePrintReceivedEvent):
 
 @wrap_event
 class TextEvent(BasePrintReceivedEvent):
-    content: Optional[Union[str, int, float, bool, list[dict[str, Union[str, dict[str, Any]]]]]] = None  # type: ignore [assignment]
+    content: str | int | float | bool | list[dict[str, str | dict[str, Any]]] | None = None  # type: ignore [assignment]
 
     @classmethod
     @require_optional_import("PIL", "unknown")
@@ -210,8 +211,8 @@ class TextEvent(BasePrintReceivedEvent):
     @field_validator("content", mode="before")
     @classmethod
     def validate_and_encode_content(
-        cls, content: Optional[Union[str, int, float, bool, list[dict[str, Union[str, dict[str, Any]]]]]]
-    ) -> Optional[Union[str, int, float, bool, list[dict[str, Union[str, dict[str, Any]]]]]]:
+        cls, content: str | int | float | bool | list[dict[str, str | dict[str, Any]]] | None
+    ) -> str | int | float | bool | list[dict[str, str | dict[str, Any]]] | None:
         if not IS_PIL_AVAILABLE:
             return content
 
@@ -224,7 +225,7 @@ class TextEvent(BasePrintReceivedEvent):
 
         return content
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
         super().print(f)
 
@@ -235,8 +236,8 @@ class TextEvent(BasePrintReceivedEvent):
 
 
 def create_received_event_model(
-    *, uuid: Optional[UUID] = None, event: dict[str, Any], sender: "Agent", recipient: "Agent"
-) -> Union[FunctionResponseEvent, ToolResponseEvent, FunctionCallEvent, ToolCallEvent, TextEvent]:
+    *, uuid: UUID | None = None, event: dict[str, Any], sender: "Agent", recipient: "Agent"
+) -> FunctionResponseEvent | ToolResponseEvent | FunctionCallEvent | ToolCallEvent | TextEvent:
     role = event.get("role")
     if role == "function":
         return FunctionResponseEvent(**event, sender=sender.name, recipient=recipient.name, uuid=uuid)
@@ -283,17 +284,17 @@ def create_received_event_model(
 
 @wrap_event
 class PostCarryoverProcessingEvent(BaseEvent):
-    carryover: Union[str, list[Union[str, dict[str, Any], Any]]]
+    carryover: str | list[str | dict[str, Any] | Any]
     message: str
     verbose: bool = False
 
     sender: str
     recipient: str
     summary_method: str
-    summary_args: Optional[dict[str, Any]] = None
-    max_turns: Optional[int] = None
+    summary_args: dict[str, Any] | None = None
+    max_turns: int | None = None
 
-    def __init__(self, *, uuid: Optional[UUID] = None, chat_info: dict[str, Any]):
+    def __init__(self, *, uuid: UUID | None = None, chat_info: dict[str, Any]):
         carryover = chat_info.get("carryover", "")
         message = chat_info.get("message")
         verbose = chat_info.get("verbose", False)
@@ -361,7 +362,7 @@ class PostCarryoverProcessingEvent(BaseEvent):
 
         return ("\n").join(print_carryover)
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
 
         print_carryover = self._process_carryover()
@@ -382,15 +383,15 @@ class PostCarryoverProcessingEvent(BaseEvent):
 
 @wrap_event
 class ClearAgentsHistoryEvent(BaseEvent):
-    agent: Optional[str] = None
-    nr_events_to_preserve: Optional[int] = None
+    agent: str | None = None
+    nr_events_to_preserve: int | None = None
 
     def __init__(
         self,
         *,
-        uuid: Optional[UUID] = None,
-        agent: Optional[Union["Agent", str]] = None,
-        nr_events_to_preserve: Optional[int] = None,
+        uuid: UUID | None = None,
+        agent: Union["Agent", str] | None = None,
+        nr_events_to_preserve: int | None = None,
     ):
         return super().__init__(
             uuid=uuid,
@@ -398,7 +399,7 @@ class ClearAgentsHistoryEvent(BaseEvent):
             nr_events_to_preserve=nr_events_to_preserve,
         )
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
 
         if self.agent:
@@ -419,16 +420,16 @@ class SpeakerAttemptSuccessfulEvent(BaseEvent):
     mentions: dict[str, int]
     attempt: int
     attempts_left: int
-    verbose: Optional[bool] = False
+    verbose: bool | None = False
 
     def __init__(
         self,
         *,
-        uuid: Optional[UUID] = None,
+        uuid: UUID | None = None,
         mentions: dict[str, int],
         attempt: int,
         attempts_left: int,
-        select_speaker_auto_verbose: Optional[bool] = False,
+        select_speaker_auto_verbose: bool | None = False,
     ):
         super().__init__(
             uuid=uuid,
@@ -448,7 +449,7 @@ class SpeakerAttemptSuccessfulEvent(BaseEvent):
             "select_speaker_auto_verbose": self.verbose,
         }
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
 
         selected_agent_name = next(iter(self.mentions))
@@ -466,16 +467,16 @@ class SpeakerAttemptFailedMultipleAgentsEvent(BaseEvent):
     mentions: dict[str, int]
     attempt: int
     attempts_left: int
-    verbose: Optional[bool] = False
+    verbose: bool | None = False
 
     def __init__(
         self,
         *,
-        uuid: Optional[UUID] = None,
+        uuid: UUID | None = None,
         mentions: dict[str, int],
         attempt: int,
         attempts_left: int,
-        select_speaker_auto_verbose: Optional[bool] = False,
+        select_speaker_auto_verbose: bool | None = False,
     ):
         super().__init__(
             uuid=uuid,
@@ -495,7 +496,7 @@ class SpeakerAttemptFailedMultipleAgentsEvent(BaseEvent):
             "select_speaker_auto_verbose": self.verbose,
         }
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
 
         f(
@@ -512,16 +513,16 @@ class SpeakerAttemptFailedNoAgentsEvent(BaseEvent):
     mentions: dict[str, int]
     attempt: int
     attempts_left: int
-    verbose: Optional[bool] = False
+    verbose: bool | None = False
 
     def __init__(
         self,
         *,
-        uuid: Optional[UUID] = None,
+        uuid: UUID | None = None,
         mentions: dict[str, int],
         attempt: int,
         attempts_left: int,
-        select_speaker_auto_verbose: Optional[bool] = False,
+        select_speaker_auto_verbose: bool | None = False,
     ):
         super().__init__(
             uuid=uuid,
@@ -541,7 +542,7 @@ class SpeakerAttemptFailedNoAgentsEvent(BaseEvent):
             "select_speaker_auto_verbose": self.verbose,
         }
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
 
         f(
@@ -557,15 +558,15 @@ class SpeakerAttemptFailedNoAgentsEvent(BaseEvent):
 class GroupChatResumeEvent(BaseEvent):
     last_speaker_name: str
     events: list[LLMMessageType]
-    verbose: Optional[bool] = False
+    verbose: bool | None = False
 
     def __init__(
         self,
         *,
-        uuid: Optional[UUID] = None,
+        uuid: UUID | None = None,
         last_speaker_name: str,
         events: list["LLMMessageType"],
-        silent: Optional[bool] = False,
+        silent: bool | None = False,
     ):
         super().__init__(uuid=uuid, last_speaker_name=last_speaker_name, events=events, verbose=not silent)
 
@@ -578,7 +579,7 @@ class GroupChatResumeEvent(BaseEvent):
             "silent": not self.verbose,
         }
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
 
         f(
@@ -591,16 +592,16 @@ class GroupChatResumeEvent(BaseEvent):
 @wrap_event
 class GroupChatRunChatEvent(BaseEvent):
     speaker: str
-    verbose: Optional[bool] = False
+    verbose: bool | None = False
 
-    def __init__(self, *, uuid: Optional[UUID] = None, speaker: Union["Agent", str], silent: Optional[bool] = False):
+    def __init__(self, *, uuid: UUID | None = None, speaker: Union["Agent", str], silent: bool | None = False):
         super().__init__(uuid=uuid, speaker=speaker.name if hasattr(speaker, "name") else speaker, verbose=not silent)
 
     @model_serializer
     def serialize_model(self) -> dict[str, Any]:
         return {"uuid": self.uuid, "speaker": self.speaker, "silent": not self.verbose}
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
 
         f(colored(f"\nNext speaker: {self.speaker}\n", "green"), flush=True)
@@ -617,9 +618,9 @@ class TerminationAndHumanReplyNoInputEvent(BaseEvent):
     def __init__(
         self,
         *,
-        uuid: Optional[UUID] = None,
+        uuid: UUID | None = None,
         no_human_input_msg: str,
-        sender: Optional[Union["Agent", str]] = None,
+        sender: Union["Agent", str] | None = None,
         recipient: Union["Agent", str],
     ):
         sender = sender or "No sender"
@@ -630,7 +631,7 @@ class TerminationAndHumanReplyNoInputEvent(BaseEvent):
             recipient=recipient.name if hasattr(recipient, "name") else recipient,
         )
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
 
         f(colored(f"\n>>>>>>>> {self.no_human_input_msg}", "red"), flush=True)
@@ -645,9 +646,9 @@ class UsingAutoReplyEvent(BaseEvent):
     def __init__(
         self,
         *,
-        uuid: Optional[UUID] = None,
+        uuid: UUID | None = None,
         human_input_mode: str,
-        sender: Optional[Union["Agent", str]] = None,
+        sender: Union["Agent", str] | None = None,
         recipient: Union["Agent", str],
     ):
         sender = sender or "No sender"
@@ -658,7 +659,7 @@ class UsingAutoReplyEvent(BaseEvent):
             recipient=recipient.name if hasattr(recipient, "name") else recipient,
         )
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
 
         f(colored("\n>>>>>>>> USING AUTO REPLY...", "red"), flush=True)
@@ -669,19 +670,25 @@ class TerminationEvent(BaseEvent):
     """When a workflow termination condition is met"""
 
     termination_reason: str
+    sender: str
+    recipient: str | None = None
 
     def __init__(
         self,
         *,
-        uuid: Optional[UUID] = None,
+        uuid: UUID | None = None,
+        sender: Union["Agent", str],
+        recipient: Union["Agent", str] | None = None,
         termination_reason: str,
     ):
         super().__init__(
             uuid=uuid,
             termination_reason=termination_reason,
+            sender=sender.name if hasattr(sender, "name") else sender,
+            recipient=recipient.name if hasattr(recipient, "name") else recipient if recipient else None,
         )
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
 
         f(colored(f"\n>>>>>>>> TERMINATING RUN ({str(self.uuid)}): {self.termination_reason}", "red"), flush=True)
@@ -697,7 +704,7 @@ class ExecuteCodeBlockEvent(BaseEvent):
     def __init__(
         self,
         *,
-        uuid: Optional[UUID] = None,
+        uuid: UUID | None = None,
         code: str,
         language: str,
         code_block_count: int,
@@ -711,7 +718,7 @@ class ExecuteCodeBlockEvent(BaseEvent):
             recipient=recipient.name if hasattr(recipient, "name") else recipient,
         )
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
 
         f(
@@ -726,16 +733,16 @@ class ExecuteCodeBlockEvent(BaseEvent):
 @wrap_event
 class ExecuteFunctionEvent(BaseEvent):
     func_name: str
-    call_id: Optional[str] = None
+    call_id: str | None = None
     arguments: dict[str, Any]
     recipient: str
 
     def __init__(
         self,
         *,
-        uuid: Optional[UUID] = None,
+        uuid: UUID | None = None,
         func_name: str,
-        call_id: Optional[str] = None,
+        call_id: str | None = None,
         arguments: dict[str, Any],
         recipient: Union["Agent", str],
     ):
@@ -747,7 +754,7 @@ class ExecuteFunctionEvent(BaseEvent):
             recipient=recipient.name if hasattr(recipient, "name") else recipient,
         )
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
 
         f(
@@ -762,8 +769,8 @@ class ExecuteFunctionEvent(BaseEvent):
 @wrap_event
 class ExecutedFunctionEvent(BaseEvent):
     func_name: str
-    call_id: Optional[str] = None
-    arguments: Optional[dict[str, Any]]
+    call_id: str | None = None
+    arguments: dict[str, Any] | None
     content: Any
     recipient: str
     is_exec_success: bool = True
@@ -771,10 +778,10 @@ class ExecutedFunctionEvent(BaseEvent):
     def __init__(
         self,
         *,
-        uuid: Optional[UUID] = None,
+        uuid: UUID | None = None,
         func_name: str,
-        call_id: Optional[str] = None,
-        arguments: Optional[dict[str, Any]],
+        call_id: str | None = None,
+        arguments: dict[str, Any] | None,
         content: Any,
         recipient: Union["Agent", str],
         is_exec_success: bool = True,
@@ -789,7 +796,7 @@ class ExecutedFunctionEvent(BaseEvent):
         )
         self.is_exec_success = is_exec_success
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
 
         f(
@@ -803,13 +810,13 @@ class ExecutedFunctionEvent(BaseEvent):
 
 @wrap_event
 class SelectSpeakerEvent(BaseEvent):
-    agents: Optional[list[str]] = None
+    agents: list[str] | None = None
 
-    def __init__(self, *, uuid: Optional[UUID] = None, agents: Optional[list[Union["Agent", str]]] = None):
+    def __init__(self, *, uuid: UUID | None = None, agents: list[Union["Agent", str]] | None = None):
         agents = [agent.name if hasattr(agent, "name") else agent for agent in agents] if agents else None
         super().__init__(uuid=uuid, agents=agents)
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
 
         f("Please select the next speaker from the following list:")
@@ -821,15 +828,13 @@ class SelectSpeakerEvent(BaseEvent):
 @wrap_event
 class SelectSpeakerTryCountExceededEvent(BaseEvent):
     try_count: int
-    agents: Optional[list[str]] = None
+    agents: list[str] | None = None
 
-    def __init__(
-        self, *, uuid: Optional[UUID] = None, try_count: int, agents: Optional[list[Union["Agent", str]]] = None
-    ):
+    def __init__(self, *, uuid: UUID | None = None, try_count: int, agents: list[Union["Agent", str]] | None = None):
         agents = [agent.name if hasattr(agent, "name") else agent for agent in agents] if agents else None
         super().__init__(uuid=uuid, try_count=try_count, agents=agents)
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
 
         f(f"You have tried {self.try_count} times. The next speaker will be selected automatically.")
@@ -837,13 +842,13 @@ class SelectSpeakerTryCountExceededEvent(BaseEvent):
 
 @wrap_event
 class SelectSpeakerInvalidInputEvent(BaseEvent):
-    agents: Optional[list[str]] = None
+    agents: list[str] | None = None
 
-    def __init__(self, *, uuid: Optional[UUID] = None, agents: Optional[list[Union["Agent", str]]] = None):
+    def __init__(self, *, uuid: UUID | None = None, agents: list[Union["Agent", str]] | None = None):
         agents = [agent.name if hasattr(agent, "name") else agent for agent in agents] if agents else None
         super().__init__(uuid=uuid, agents=agents)
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
 
         f(f"Invalid input. Please enter a number between 1 and {len(self.agents or [])}.")
@@ -855,9 +860,7 @@ class ClearConversableAgentHistoryEvent(BaseEvent):
     recipient: str
     no_events_preserved: int
 
-    def __init__(
-        self, *, uuid: Optional[UUID] = None, agent: Union["Agent", str], no_events_preserved: Optional[int] = None
-    ):
+    def __init__(self, *, uuid: UUID | None = None, agent: Union["Agent", str], no_events_preserved: int | None = None):
         super().__init__(
             uuid=uuid,
             agent=agent.name if hasattr(agent, "name") else agent,
@@ -873,7 +876,7 @@ class ClearConversableAgentHistoryEvent(BaseEvent):
             "no_events_preserved": self.no_events_preserved,
         }
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
 
         for _ in range(self.no_events_preserved):
@@ -884,13 +887,13 @@ class ClearConversableAgentHistoryEvent(BaseEvent):
 class ClearConversableAgentHistoryWarningEvent(BaseEvent):
     recipient: str
 
-    def __init__(self, *, uuid: Optional[UUID] = None, recipient: Union["Agent", str]):
+    def __init__(self, *, uuid: UUID | None = None, recipient: Union["Agent", str]):
         super().__init__(
             uuid=uuid,
             recipient=recipient.name if hasattr(recipient, "name") else recipient,
         )
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
 
         f(
@@ -911,9 +914,9 @@ class GenerateCodeExecutionReplyEvent(BaseEvent):
     def __init__(
         self,
         *,
-        uuid: Optional[UUID] = None,
+        uuid: UUID | None = None,
         code_blocks: list[Union["CodeBlock", str]],
-        sender: Optional[Union["Agent", str]] = None,
+        sender: Union["Agent", str] | None = None,
         recipient: Union["Agent", str],
     ):
         code_blocks = [
@@ -928,7 +931,7 @@ class GenerateCodeExecutionReplyEvent(BaseEvent):
             recipient=recipient.name if hasattr(recipient, "name") else recipient,
         )
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
 
         num_code_blocks = len(self.code_blocks)
@@ -954,10 +957,10 @@ class GenerateCodeExecutionReplyEvent(BaseEvent):
 class ConversableAgentUsageSummaryNoCostIncurredEvent(BaseEvent):
     recipient: str
 
-    def __init__(self, *, uuid: Optional[UUID] = None, recipient: Union["Agent", str]):
+    def __init__(self, *, uuid: UUID | None = None, recipient: Union["Agent", str]):
         super().__init__(uuid=uuid, recipient=recipient.name if hasattr(recipient, "name") else recipient)
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
 
         f(f"No cost incurred from agent '{self.recipient}'.")
@@ -967,10 +970,10 @@ class ConversableAgentUsageSummaryNoCostIncurredEvent(BaseEvent):
 class ConversableAgentUsageSummaryEvent(BaseEvent):
     recipient: str
 
-    def __init__(self, *, uuid: Optional[UUID] = None, recipient: Union["Agent", str]):
+    def __init__(self, *, uuid: UUID | None = None, recipient: Union["Agent", str]):
         super().__init__(uuid=uuid, recipient=recipient.name if hasattr(recipient, "name") else recipient)
 
-    def print(self, f: Optional[Callable[..., Any]] = None) -> None:
+    def print(self, f: Callable[..., Any] | None = None) -> None:
         f = f or print
 
         f(f"Agent '{self.recipient}':")
@@ -980,7 +983,7 @@ class ConversableAgentUsageSummaryEvent(BaseEvent):
 class InputRequestEvent(BaseEvent):
     prompt: str
     password: bool = False
-    respond: Optional[Callable[[str], None]] = None
+    respond: Callable[[str], None] | None = None
 
     type: str = "input_request"
 
@@ -1009,5 +1012,5 @@ class RunCompletionEvent(BaseEvent):
     summary: str
     history: list[LLMMessageType]
     cost: dict[str, Any]
-    last_speaker: Optional[str]
-    context_variables: Optional[ContextVariables] = None
+    last_speaker: str | None
+    context_variables: ContextVariables | None = None
