@@ -51,15 +51,15 @@ import re
 import time
 import warnings
 from io import BytesIO
-from typing import Any, Literal, Optional, Type, Union
+from typing import Any, Literal
 
 import requests
-from packaging import version
 from pydantic import BaseModel, Field
+from typing_extensions import Unpack
 
 from ..import_utils import optional_import_block, require_optional_import
 from ..json_utils import resolve_json_references
-from ..llm_config import LLMConfigEntry, register_llm_config
+from ..llm_config.entry import LLMConfigEntry, LLMConfigEntryDict
 from .client_utils import FormatterProtocol
 from .gemini_types import ToolConfig
 from .oai_models import ChatCompletion, ChatCompletionMessage, ChatCompletionMessageToolCall, Choice, CompletionUsage
@@ -69,6 +69,7 @@ with optional_import_block():
     import vertexai
     from PIL import Image
     from google.auth.credentials import Credentials
+    from google.genai import types
     from google.genai.types import (
         Content,
         FinishReason,
@@ -101,19 +102,34 @@ with optional_import_block():
 logger = logging.getLogger(__name__)
 
 
-@register_llm_config
+class GeminiEntryDict(LLMConfigEntryDict, total=False):
+    api_type: Literal["google"]
+
+    project_id: str | None
+    location: str | None
+    google_application_credentials: str | None
+    credentials: Any | str | None
+    stream: bool
+    safety_settings: list[dict[str, Any]] | dict[str, Any] | None
+    price: list[float] | None
+    tool_config: ToolConfig | None
+    proxy: str | None
+
+
 class GeminiLLMConfigEntry(LLMConfigEntry):
     api_type: Literal["google"] = "google"
-    project_id: Optional[str] = None
-    location: Optional[str] = None
+    project_id: str | None = None
+    location: str | None = None
     # google_application_credentials points to the path of the JSON Keyfile
-    google_application_credentials: Optional[str] = None
+    google_application_credentials: str | None = None
     # credentials is a google.auth.credentials.Credentials object
-    credentials: Optional[Union[Any, str]] = None
+    credentials: Any | str | None = None
     stream: bool = False
-    safety_settings: Optional[Union[list[dict[str, Any]], dict[str, Any]]] = None
-    price: Optional[list[float]] = Field(default=None, min_length=2, max_length=2)
-    tool_config: Optional[ToolConfig] = None
+    safety_settings: list[dict[str, Any]] | dict[str, Any] | None = None
+    price: list[float] | None = Field(default=None, min_length=2, max_length=2)
+    tool_config: ToolConfig | None = None
+    proxy: str | None = None
+    """A valid HTTP(S) proxy URL"""
 
     def create_client(self):
         raise NotImplementedError("GeminiLLMConfigEntry.create_client() is not implemented.")
@@ -127,6 +143,7 @@ class GeminiClient:
     PARAMS_MAPPING = {
         "max_tokens": "max_output_tokens",
         # "n": "candidate_count", # Gemini supports only `n=1`
+        "seed": "seed",
         "stop_sequences": "stop_sequences",
         "temperature": "temperature",
         "top_p": "top_p",
@@ -134,7 +151,7 @@ class GeminiClient:
         "max_output_tokens": "max_output_tokens",
     }
 
-    def _initialize_vertexai(self, **params):
+    def _initialize_vertexai(self, **params: Unpack[GeminiEntryDict]):
         if "google_application_credentials" in params:
             # Path to JSON Keyfile
             os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = params["google_application_credentials"]
@@ -179,9 +196,10 @@ class GeminiClient:
             )
 
         self.api_version = kwargs.get("api_version")
+        self.proxy = kwargs.get("proxy")
 
         # Store the response format, if provided (for structured outputs)
-        self._response_format: Optional[type[BaseModel]] = None
+        self._response_format: type[BaseModel] | None = None
 
     def message_retrieval(self, response) -> list:
         """Retrieve and return a list of strings or a list of Choice.Message from the response.
@@ -237,8 +255,14 @@ class GeminiClient:
                 "See this [LLM configuration tutorial](https://docs.ag2.ai/latest/docs/user-guide/basic-concepts/llm-configuration/) for more details."
             )
 
-        params.get("api_type", "google")  # not used
-        http_options = {"api_version": self.api_version} if self.api_version else None
+        http_options = types.HttpOptions()
+        if proxy := params.get("proxy", self.proxy):
+            http_options.client_args = {"proxy": proxy}
+            http_options.async_client_args = {"proxy": proxy}
+
+        if self.api_version:
+            http_options.api_version = self.api_version
+
         messages = params.get("messages", [])
         stream = params.get("stream", False)
         n_response = params.get("n", 1)
@@ -332,6 +356,12 @@ class GeminiClient:
                 recitation_part = Part(text="Unsuccessful Finish Reason: RECITATION")
                 parts = [recitation_part]
                 error_finish_reason = "content_filter"  # As per available finish_reason in Choice
+            elif not response.candidates[0].content or not response.candidates[0].content.parts:
+                error_part = Part(
+                    text=f"Unsuccessful Finish Reason: ({str(response.candidates[0].finish_reason)}) NO CONTENT RETURNED"
+                )
+                parts = [error_part]
+                error_finish_reason = "content_filter"  # No other option in Choice in chat_completion.py
             else:
                 parts = response.candidates[0].content.parts
         elif isinstance(response, VertexAIGenerationResponse):  # or hasattr(response, "candidates"):
@@ -570,16 +600,19 @@ class GeminiClient:
 
             if part_type == "text":
                 rst.append(
-                    VertexAIContent(parts=parts, role=role)
-                    if self.use_vertexai
-                    else rst.append(Content(parts=parts, role=role))
+                    VertexAIContent(parts=parts, role=role) if self.use_vertexai else Content(parts=parts, role=role)
                 )
-            elif part_type == "tool" or part_type == "tool_call":
-                role = "function" if version.parse(genai.__version__) < version.parse("1.4.0") else "user"
+            elif part_type == "tool_call":
+                # Function calls should be from the model/assistant
+                role = "model"
                 rst.append(
-                    VertexAIContent(parts=parts, role=role)
-                    if self.use_vertexai
-                    else rst.append(Content(parts=parts, role=role))
+                    VertexAIContent(parts=parts, role=role) if self.use_vertexai else Content(parts=parts, role=role)
+                )
+            elif part_type == "tool":
+                # Function responses should be from the user
+                role = "user"
+                rst.append(
+                    VertexAIContent(parts=parts, role=role) if self.use_vertexai else Content(parts=parts, role=role)
                 )
             elif part_type == "image":
                 # Image has multiple parts, some can be text and some can be image based
@@ -599,14 +632,14 @@ class GeminiClient:
                     rst.append(
                         VertexAIContent(parts=text_parts, role=role)
                         if self.use_vertexai
-                        else rst.append(Content(parts=text_parts, role=role))
+                        else Content(parts=text_parts, role=role)
                     )
 
                 if len(image_parts) > 0:
                     rst.append(
                         VertexAIContent(parts=image_parts, role=role)
                         if self.use_vertexai
-                        else rst.append(Content(parts=image_parts, role=role))
+                        else Content(parts=image_parts, role=role)
                     )
 
             if len(rst) != 0 and rst[-1] is None:
@@ -661,9 +694,7 @@ class GeminiClient:
 
     @staticmethod
     def _convert_type_null_to_nullable(schema: Any) -> Any:
-        """
-        Recursively converts all occurrences of {"type": "null"} to {"nullable": True} in a schema.
-        """
+        """Recursively converts all occurrences of {"type": "null"} to {"nullable": True} in a schema."""
         if isinstance(schema, dict):
             # If schema matches {"type": "null"}, replace it
             if schema == {"type": "null"}:
@@ -899,18 +930,25 @@ def calculate_gemini_cost(use_vertexai: bool, input_tokens: int, output_tokens:
         # https://cloud.google.com/vertex-ai/generative-ai/pricing#vertex-ai-pricing
 
         if (
-            "gemini-2.5-pro-preview-03-25" in model_name
-            or "gemini-2.5-pro-exp-03-25" in model_name
+            model_name == "gemini-2.5-pro"
+            or "gemini-2.5-pro-preview-06-05" in model_name
             or "gemini-2.5-pro-preview-05-06" in model_name
+            or "gemini-2.5-pro-preview-03-25" in model_name
         ):
             if up_to_200k:
                 return total_cost_mil(1.25, 10)
             else:
                 return total_cost_mil(2.5, 15)
 
-        elif "gemini-2.5-flash-preview-04-17" in model_name:
+        elif "gemini-2.5-flash" in model_name:
+            return total_cost_mil(0.3, 2.5)
+
+        elif "gemini-2.5-flash-preview-04-17" in model_name or "gemini-2.5-flash-preview-05-20" in model_name:
             return total_cost_mil(0.15, 0.6)  # NON-THINKING OUTPUT PRICE, $3 FOR THINKING!
 
+        elif "gemini-2.5-flash-lite-preview-06-17" in model_name:
+            return total_cost_mil(0.1, 0.4)
+
         elif "gemini-2.0-flash-lite" in model_name:
             return total_cost_mil(0.075, 0.3)
 
@@ -943,9 +981,10 @@ def calculate_gemini_cost(use_vertexai: bool, input_tokens: int, output_tokens:
         # Non-Vertex AI pricing
 
         if (
-            "gemini-2.5-pro-preview-03-25" in model_name
-            or "gemini-2.5-pro-exp-03-25" in model_name
+            model_name == "gemini-2.5-pro"
+            or "gemini-2.5-pro-preview-06-05" in model_name
             or "gemini-2.5-pro-preview-05-06" in model_name
+            or "gemini-2.5-pro-preview-03-25" in model_name
         ):
             # https://ai.google.dev/gemini-api/docs/pricing#gemini-2.5-pro-preview
             if up_to_200k:
@@ -953,10 +992,18 @@ def calculate_gemini_cost(use_vertexai: bool, input_tokens: int, output_tokens:
             else:
                 return total_cost_mil(2.5, 15)
 
-        elif "gemini-2.5-flash-preview-04-17" in model_name:
+        elif "gemini-2.5-flash" in model_name:
+            # https://ai.google.dev/gemini-api/docs/pricing#gemini-2.5-flash
+            return total_cost_mil(0.3, 2.5)
+
+        elif "gemini-2.5-flash-preview-04-17" in model_name or "gemini-2.5-flash-preview-05-20" in model_name:
             # https://ai.google.dev/gemini-api/docs/pricing#gemini-2.5-flash
             return total_cost_mil(0.15, 0.6)
 
+        elif "gemini-2.5-flash-lite-preview-06-17" in model_name:
+            # https://ai.google.dev/gemini-api/docs/pricing#gemini-2.5-flash-lite
+            return total_cost_mil(0.1, 0.4)
+
         elif "gemini-2.0-flash-lite" in model_name:
             # https://ai.google.dev/gemini-api/docs/pricing#gemini-2.0-flash-lite
             return total_cost_mil(0.075, 0.3)
