@@ -3,9 +3,11 @@
 # SPDX-License-Identifier: Apache-2.0
 
 from .browser_use import BrowserUseTool
+from .code_execution import PythonCodeExecutionTool
 from .crawl4ai import Crawl4AITool
 from .deep_research import DeepResearchTool
 from .duckduckgo import DuckDuckGoSearchTool
+from .firecrawl import FirecrawlTool
 from .google_search import GoogleSearchTool, YoutubeSearchTool
 from .messageplatform import (
     DiscordRetrieveTool,
@@ -18,6 +20,7 @@ from .messageplatform import (
 )
 from .perplexity import PerplexitySearchTool
 from .reliable import ReliableTool, ReliableToolError, SuccessfulExecutionParameters, ToolExecutionDetails
+from .searxng import SearxngSearchTool
 from .tavily import TavilySearchTool
 from .web_search_preview import WebSearchPreviewTool
 from .wikipedia import WikipediaPageLoadTool, WikipediaQueryRunTool
@@ -29,10 +32,13 @@ __all__ = [
     "DiscordRetrieveTool",
     "DiscordSendTool",
     "DuckDuckGoSearchTool",
+    "FirecrawlTool",
     "GoogleSearchTool",
     "PerplexitySearchTool",
+    "PythonCodeExecutionTool",
     "ReliableTool",
     "ReliableToolError",
+    "SearxngSearchTool",
     "SlackRetrieveRepliesTool",
     "SlackRetrieveTool",
     "SlackSendTool",
